#!/usr/bin/fades

"""Remove all dirs/files that are older than N days.

It just checks ~/temp and first level of directories or files, no deeper.
"""

import pathlib
import shutil
import sys
from datetime import datetime, timedelta

import pytest  # fades

BASEDIR = pathlib.Path.home() / "temp"


def remove(node):
    """Remove the node.

    If it's a file or a symlink to a file, remove it.

    If it's a directory go inside it and repeat. If it's a symlink to a directory
    outside BASEDIR, do NOT follow it.
    """
    if node.is_symlink() or node.is_file():
        node.unlink()
    elif node.is_dir():
        shutil.rmtree(node)
    else:
        print("     NOT REALLY! unknown node")


def main(ndays):
    """Main entry point."""
    threshold = datetime.now() - timedelta(days=ndays)

    for node in BASEDIR.iterdir():
        mtime = datetime.fromtimestamp(node.stat().st_mtime)
        if mtime <= threshold:
            print("Removing!", mtime)
            print("    ", repr(str(node)))
            remove(node)


# -- tests

def test_remove_simple_file(tmp_path):
    node = tmp_path / "somefile"
    node.write_text("asd")

    remove(node)
    assert not node.exists()


def test_remove_empty_dir(tmp_path):
    node = tmp_path / "somedir"
    node.mkdir()

    remove(node)
    assert not node.exists()


def test_remove_nonempty_dir(tmp_path):
    node = tmp_path / "somedir"
    node.mkdir()
    (node / "subfile1").touch()
    (node / "subfile2").write_text("asd")
    (node / "subdir").mkdir()
    (node / "subdir" / "subfile3").write_text("qew")

    remove(node)
    assert not node.exists()


def test_remove_symlink_file_internal(tmp_path):
    basedir = tmp_path / "basedir"
    basedir.mkdir()
    node1 = basedir / "somefile1"
    node1.write_text("test")
    node2 = basedir / "somefile2"
    node2.symlink_to(node1)

    remove(node2)
    assert not node2.exists()
    assert node1.exists()


def test_remove_symlink_file_external(tmp_path):
    node_out = tmp_path / "somefile1"
    node_out.write_text("test")
    basedir = tmp_path / "basedir"
    basedir.mkdir()
    node_in = basedir / "somefile2"
    node_in.symlink_to(node_out)

    remove(basedir)
    assert not basedir.exists()
    assert node_out.exists()


def test_remove_symlink_dir_internal(tmp_path):
    basedir = tmp_path / "basedir"
    basedir.mkdir()
    node1 = basedir / "somedir1"
    node1.mkdir()
    node2 = basedir / "somedir2"
    node2.symlink_to(node1)

    remove(basedir)
    assert not basedir.exists()


def test_remove_symlink_dir_external(tmp_path):
    dir_out = tmp_path / "somedir1"
    dir_out.mkdir()
    file_out = dir_out / "file1"
    file_out.write_text(":)")

    basedir = tmp_path / "basedir"
    basedir.mkdir()
    dir_in = basedir / "somedir2"
    dir_in.symlink_to(dir_out)

    remove(basedir)
    assert not basedir.exists()
    assert dir_out.exists()
    assert file_out.exists()


if "--autotest" in sys.argv:
    pytest.main([sys.argv[0]])
else:
    try:
        ndays = int(sys.argv[1])
    except (IndexError, ValueError):
        print("USAGE:  clean-old-temp.py [--autotest] <days>")
        exit(1)
    main(ndays)
